import 'package:flutter/material.dart';
import 'package:fl_chart/fl_chart.dart';
import 'package:factory_signage_app/const/colors.dart';
import 'package:factory_signage_app/const/dimens.dart';
import 'package:factory_signage_app/const/string.dart';

// Declare a variable in the field since the size of the current widget
// cannot be passed to getTitlesWidget as an argument
// ignore: must_be_immutable
class ProgressGraph extends StatelessWidget {
  ProgressGraph(
    this.barDateY, {
    super.key,
    this.size = const Size(double.infinity, double.infinity),
    this.maxY,
    this.interval,
  });

  final List<int> barDateY;
  final Size size;
  final double? maxY;
  final double? interval;
  Size widgetSize = const Size(0, 0);

  @override
  Widget build(BuildContext context) {
    widgetSize = MediaQuery.of(context).size;

    return SizedBox(
      width: size.width,
      height: size.height,
      child: progressGraph(widgetSize),
    );
  }

  Widget progressGraph(Size widgetSize) {
    return Container(
      alignment: Alignment.center,
      color: graphBackgroundColor,
      child: BarChart(
        BarChartData(
          maxY: maxY,
          barGroups: [
            for (int i = 0; i < barDateY.length; i++) ...{
              generateGroupData(widgetSize, i, barDateY[i])
            }
          ],
          titlesData: FlTitlesData(
            leftTitles: AxisTitles(
              sideTitles: SideTitles(
                showTitles: true,
                reservedSize: graphLeftReservedSizeRatio * widgetSize.height,
                interval: interval,
                getTitlesWidget: leftTitles,
              ),
            ),
            topTitles: AxisTitles(
              sideTitles: SideTitles(
                showTitles: false,
              ),
            ),
            rightTitles: AxisTitles(
              sideTitles: SideTitles(
                showTitles: false,
              ),
            ),
            bottomTitles: AxisTitles(
              sideTitles: SideTitles(
                showTitles: true,
                reservedSize: graphBottomReservedSizeRatio * widgetSize.height,
                getTitlesWidget: bottomTitles,
              ),
            ),
          ),
        ),
      ),
    );
  }

  BarChartGroupData generateGroupData(Size widgetSize, int x, int y) {
    return BarChartGroupData(
      x: x,
      barRods: [
        BarChartRodData(
          toY: y.toDouble(),
          color: barChartRodColor,
          width: barChartRodWidth * widgetSize.width,
          borderRadius: const BorderRadius.vertical(),
        ),
      ],
    );
  }

  Widget leftTitles(double value, TitleMeta meta) {
    if (value == meta.min || value == meta.max) {
      return Container();
    }
    return SideTitleWidget(
      axisSide: meta.axisSide,
      space: graphLeftTitleSpaceRatio * widgetSize.width,
      child: Text(
        // Cast to int type and then convert to String type
        // because decimal point is displayed
        value.toInt().toString(),
        style: TextStyle(
          fontSize: graphLeftTitleFontSizeRatio * widgetSize.height,
          fontWeight: FontWeight.bold,
        ),
        textAlign: TextAlign.right,
      ),
    );
  }

  Widget bottomTitles(double value, TitleMeta meta) {
    return SideTitleWidget(
      axisSide: meta.axisSide,
      space: graphBottomSpaceRatio * widgetSize.height,
      child: Text(
        lineNames[(value).toInt()],
        style: TextStyle(
          fontSize: graphBottomTitleFontSizeRatio * widgetSize.height,
          fontWeight: FontWeight.bold,
        ),
      ),
    );
  }
}
